@@ -141,7 +141,7 @@ NOTES:
         meth_dfs = []
         unmeth_dfs = []
         for part in test_parts:
-            if 'meth_values' in part and 'noob' not in part:
+            if 'meth_values' in part and 'noob' not in part and 'unmeth_values' not in part:
                 meth_dfs.append( pd.read_pickle(part) )
             if 'unmeth_values' in part and 'noob' not in part:
                 unmeth_dfs.append( pd.read_pickle(part) )
@@ -244,6 +244,10 @@ NOTES:
                     sample_name = fname.replace('_processed.csv.gz','')
                 elif '_processed.csv' in fname:
                     sample_name = fname.replace('_processed.csv','')
+                elif '.processed.csv.gz' in fname:
+                    sample_name = fname.replace('.processed.csv.gz','')
+                elif '.processed.csv' in fname:
+                    sample_name = fname.replace('.processed.csv','')
                 else:
                     sample_name = ''
                 # FUTURE TODO: if sample_sheet or meta_data supplied, fill in with proper sample_names here
